@@ -14,8 +14,8 @@ import (
 	"github.com/gorilla/mux"
 	"github.com/prometheus/client_golang/prometheus"
 	"github.com/skupperproject/skupper/api/types"
-	"github.com/skupperproject/skupper/client"
 	"github.com/skupperproject/skupper/pkg/config"
+	"github.com/skupperproject/skupper/pkg/domain/podman"
 	"github.com/skupperproject/skupper/pkg/kube"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/wait"
@@ -487,69 +487,96 @@ var defaultRetry = wait.Backoff{
 	Jitter:   0.1,
 }
 
-func (fc *FlowCollector) updateNetworkStatus() error {
+var netUpdateCt int
+
+func (fc *FlowCollector) updateNetworkStatus() {
 	var err error
 	networkData := map[string]string{}
 	platform := config.GetPlatform()
-	if platform == "" || platform == types.PlatformKubernetes {
-		cli, err := client.NewClient(fc.namespace, "", "")
-		if err != nil {
-			return err
-		}
+	var sites []*SiteStatus
+	var addresses []*VanAddressRecord
 
-		var sites []*SiteStatus
-		var addresses []*VanAddressRecord
-
-		for _, address := range fc.VanAddresses {
-			fc.getAddressAdaptorCounts(address)
-			addresses = append(addresses, address)
+	for _, address := range fc.VanAddresses {
+		fc.getAddressAdaptorCounts(address)
+		addresses = append(addresses, address)
+	}
+	for _, site := range fc.Sites {
+		var routerStatus []RouterStatus
+		routers := fc.getRoutersForSite(site)
+		for _, router := range routers {
+			links := fc.getLinksForRouter(router)
+			listeners := fc.getListenersForRouter(router)
+			connectors := fc.getConnectorsForRouter(router)
+			routerStatus = append(routerStatus, RouterStatus{
+				Router:     router,
+				Links:      links,
+				Listeners:  listeners,
+				Connectors: connectors,
+			})
 		}
-		for _, site := range fc.Sites {
-			var routerStatus []RouterStatus
-			routers := fc.getRoutersForSite(site)
-			for _, router := range routers {
-				links := fc.getLinksForRouter(router)
-				listeners := fc.getListenersForRouter(router)
-				connectors := fc.getConnectorsForRouter(router)
-				routerStatus = append(routerStatus, RouterStatus{
-					Router:     router,
-					Links:      links,
-					Listeners:  listeners,
-					Connectors: connectors,
-				})
-			}
-			siteStatus := &SiteStatus{
-				Site:         site,
-				RouterStatus: routerStatus,
-			}
-			sites = append(sites, siteStatus)
+		siteStatus := &SiteStatus{
+			Site:         site,
+			RouterStatus: routerStatus,
 		}
-		sort.SliceStable(addresses, func(i, j int) bool {
-			return addresses[i].Name < addresses[j].Name
-		})
-		networkStatus := NetworkStatus{
-			Addresses: addresses,
-			Sites:     sites,
+		sites = append(sites, siteStatus)
+	}
+	sort.SliceStable(addresses, func(i, j int) bool {
+		return addresses[i].Name < addresses[j].Name
+	})
+	networkStatus := NetworkStatus{
+		Addresses: addresses,
+		Sites:     sites,
+	}
+	networkData["NetworkStatus"] = prettyPrint(networkStatus)
+
+	if platform == "" || platform == types.PlatformKubernetes {
+		if fc.kubeclient == nil { // errant configuration - means there is a bug in FlowCollector or how it was configured
+			panic("FlowCollector was not configured with a kubernetes client")
 		}
-		networkData["NetworkStatus"] = prettyPrint(networkStatus)
+		err = func() error {
+			err = retry.RetryOnConflict(defaultRetry, func() error {
+				configMap, err := kube.GetConfigMap(types.NetworkStatusConfigMapName, fc.namespace, fc.kubeclient)
+				if err != nil {
+					return err
+				}
+
+				configMap.Data = networkData
 
-		err = retry.RetryOnConflict(defaultRetry, func() error {
-			configMap, err := kube.GetConfigMap(types.NetworkStatusConfigMapName, cli.Namespace, cli.KubeClient)
-			if err != nil {
-				return err
+				_, err = fc.kubeclient.CoreV1().ConfigMaps(fc.namespace).Update(context.TODO(), configMap, metav1.UpdateOptions{})
+				if err != nil {
+					return err
+				} else {
+					netUpdateCt++
+					return nil
+				}
+			})
+			if !fc.networkStatusUp && len(networkStatus.Sites) > 0 && len(networkStatus.Sites[0].RouterStatus) > 0 {
+				fc.networkStatusUp = true
+				log.Printf("COLLECTOR: First functional network status update written after %s and %d updates\n", time.Since(fc.begin), netUpdateCt)
 			}
+			return nil
+		}()
+	} else if platform == types.PlatformPodman {
+		networkStatusHandler := &podman.NetworkStatusHandler{}
+		err = networkStatusHandler.Update(networkData["NetworkStatus"])
+	}
 
-			configMap.Data = networkData
+	if err != nil {
+		log.Printf("COLLECTOR: Error writing network status update: %v", err)
+	}
+}
 
-			_, err = cli.KubeClient.CoreV1().ConfigMaps(cli.Namespace).Update(context.TODO(), configMap, metav1.UpdateOptions{})
-			if err != nil {
-				return err
-			} else {
-				return nil
-			}
-		})
+func (fc *FlowCollector) updateEntityCounts() {
+	if fc.mode != RecordMetrics {
+		return
+	}
+	fc.metrics.activeSites.Set(float64(len(fc.Sites)))
+	fc.metrics.activeRouters.Set(float64(len(fc.Routers)))
+	_, siteNodes := fc.graph()
+	for _, node := range siteNodes {
+		fc.metrics.activeLinks.WithLabelValues(node.ID, "outgoing").Set(float64(len(node.Forward)))
+		fc.metrics.activeLinks.WithLabelValues(node.ID, "incoming").Set(float64(len(node.Backward)))
 	}
-	return err
 }
 
 func (fc *FlowCollector) addRecord(record interface{}) error {
@@ -729,6 +756,7 @@ func (fc *FlowCollector) updateRecord(record interface{}) error {
 				}
 			}
 			fc.updateLastHeard(site.Source)
+			fc.updateEntityCounts()
 		}
 	case HostRecord:
 		if host, ok := record.(HostRecord); ok {
@@ -767,6 +795,7 @@ func (fc *FlowCollector) updateRecord(record interface{}) error {
 				}
 			}
 			fc.updateLastHeard(router.Source)
+			fc.updateEntityCounts()
 		}
 	case LogEventRecord:
 		if logEvent, ok := record.(LogEventRecord); ok {
@@ -785,6 +814,7 @@ func (fc *FlowCollector) updateRecord(record interface{}) error {
 				}
 			}
 			fc.updateLastHeard(link.Source)
+			fc.updateEntityCounts()
 		}
 	case ListenerRecord:
 		if listener, ok := record.(ListenerRecord); ok {
@@ -933,8 +963,11 @@ func (fc *FlowCollector) updateRecord(record interface{}) error {
 					current.EndTime = connector.EndTime
 					if current.ProcessId != nil {
 						if process, ok := fc.Processes[*current.ProcessId]; ok {
-							process.connector = nil
-							process.ProcessBinding = &Unbound
+							if process.Name == nil || !strings.HasPrefix(*process.Name, "site-servers-") {
+								// site servers can be reused by multiple connectors - do not unbind
+								process.connector = nil
+								process.ProcessBinding = &Unbound
+							}
 						}
 					}
 					// Note a new connector can create an address but does not delete it
@@ -1193,6 +1226,59 @@ func (fc *FlowCollector) updateRecord(record interface{}) error {
 	return nil
 }
 
+type linkResponseHandler struct {
+	siteByRouterID   map[string]string
+	siteByRouterName map[string]string
+}
+
+func newLinkResponseHandler(sites map[string]*SiteRecord, routers map[string]*RouterRecord) linkResponseHandler {
+	builder := linkResponseHandler{
+		siteByRouterID:   make(map[string]string, len(routers)),
+		siteByRouterName: make(map[string]string, len(routers)),
+	}
+	for _, router := range routers {
+		_, ok := sites[router.Parent]
+		if !ok || router.Name == nil {
+			continue
+		}
+
+		builder.siteByRouterName[normalizeRouterName(*router.Name)] = router.Parent
+		builder.siteByRouterID[router.Identity] = router.Parent
+	}
+	return builder
+}
+
+func normalizeRouterName(name string) string {
+	// router names are prefixed with a routing area - so far always `0/`
+	if delim := strings.IndexRune(name, '/'); delim > -1 {
+		return name[delim+1:]
+	}
+	return name
+}
+
+func (b linkResponseHandler) handle(l LinkRecord) (linkRecordResponse, bool) {
+	var (
+		ok   bool
+		resp = linkRecordResponse{LinkRecord: l}
+	)
+	if l.Name == nil || l.Direction == nil {
+		return resp, false
+	}
+
+	if resp.SourceSiteId, ok = b.siteByRouterID[l.Parent]; !ok {
+		return resp, false
+	}
+
+	if resp.DestinationSiteId, ok = b.siteByRouterName[*l.Name]; !ok {
+		return resp, false
+	}
+	if *l.Direction == Incoming {
+		resp.SourceSiteId, resp.DestinationSiteId = resp.DestinationSiteId, resp.SourceSiteId
+	}
+
+	return resp, true
+}
+
 func (fc *FlowCollector) retrieve(request ApiRequest) (*string, error) {
 	vars := mux.Vars(request.Request)
 	url := request.Request.URL
@@ -1259,14 +1345,19 @@ func (fc *FlowCollector) retrieve(request ApiRequest) (*string, error) {
 			}
 			retrieveError = sortAndSlice(routers, &p, queryParams)
 		case "links":
-			links := []LinkRecord{}
+			links := []linkRecordResponse{}
+			linkHandler := newLinkResponseHandler(fc.Sites, fc.Routers)
 			if id, ok := vars["id"]; ok {
 				if site, ok := fc.Sites[id]; ok {
 					for _, link := range fc.Links {
 						if fc.getRecordSiteId(*link) == site.Identity {
+							lr, ok := linkHandler.handle(*link)
+							if !ok {
+								continue
+							}
 							p.TotalCount++
-							if filterRecord(*link, queryParams) && link.Base.TimeRangeValid(queryParams) {
-								links = append(links, *link)
+							if filterRecord(lr, queryParams) && link.Base.TimeRangeValid(queryParams) {
+								links = append(links, lr)
 							}
 						}
 					}
@@ -1372,14 +1463,19 @@ func (fc *FlowCollector) retrieve(request ApiRequest) (*string, error) {
 			}
 			retrieveError = sortAndSlice(flows, &p, queryParams)
 		case "links":
-			links := []LinkRecord{}
+			links := []linkRecordResponse{}
+			linkHandler := newLinkResponseHandler(fc.Sites, fc.Routers)
 			if id, ok := vars["id"]; ok {
 				if router, ok := fc.Routers[id]; ok {
 					for _, link := range fc.Links {
 						if link.Parent == router.Identity {
+							lr, ok := linkHandler.handle(*link)
+							if !ok {
+								continue
+							}
 							p.TotalCount++
-							if filterRecord(*link, queryParams) && link.Base.TimeRangeValid(queryParams) {
-								links = append(links, *link)
+							if filterRecord(lr, queryParams) && link.Base.TimeRangeValid(queryParams) {
+								links = append(links, lr)
 							}
 						}
 					}
@@ -1418,12 +1514,17 @@ func (fc *FlowCollector) retrieve(request ApiRequest) (*string, error) {
 			retrieveError = sortAndSlice(connectors, &p, queryParams)
 		}
 	case Link:
+		linkHandler := newLinkResponseHandler(fc.Sites, fc.Routers)
 		switch request.HandlerName {
 		case "list":
-			links := []LinkRecord{}
+			links := []linkRecordResponse{}
 			for _, link := range fc.Links {
-				if filterRecord(*link, queryParams) && link.Base.TimeRangeValid(queryParams) {
-					links = append(links, *link)
+				lr, ok := linkHandler.handle(*link)
+				if !ok {
+					continue
+				}
+				if filterRecord(lr, queryParams) && link.Base.TimeRangeValid(queryParams) {
+					links = append(links, lr)
 				}
 			}
 			p.TotalCount = len(fc.Links)
@@ -1431,8 +1532,11 @@ func (fc *FlowCollector) retrieve(request ApiRequest) (*string, error) {
 		case "item":
 			if id, ok := vars["id"]; ok {
 				if link, ok := fc.Links[id]; ok {
-					p.Count = 1
-					p.Results = link
+					lr, ok := linkHandler.handle(*link)
+					if ok {
+						p.Count = 1
+						p.Results = lr
+					}
 				}
 			}
 		}
@@ -2268,6 +2372,7 @@ func (fc *FlowCollector) reconcileFlowRecords() error {
 			if flow.SourceHost != nil {
 				if connector, ok := fc.Connectors[flow.Parent]; ok {
 					flow.Place = serverSide
+					flow.Protocol = connector.Protocol
 					if connector.ProcessId != nil && connector.AddressId != nil {
 						flow.Process = connector.ProcessId
 						if process, ok := fc.Processes[*flow.Process]; ok {
@@ -2281,6 +2386,7 @@ func (fc *FlowCollector) reconcileFlowRecords() error {
 					}
 				} else if listener, ok := fc.Listeners[flow.Parent]; ok {
 					flow.Place = clientSide
+					flow.Protocol = listener.Protocol
 					if listener.AddressId != nil {
 						found := false
 						for _, process := range fc.Processes {
@@ -2304,6 +2410,7 @@ func (fc *FlowCollector) reconcileFlowRecords() error {
 				if l4Flow.Process != nil {
 					flow.Process = l4Flow.Process
 					flow.ProcessName = l4Flow.ProcessName
+					flow.Protocol = fc.getFlowProtocol(l4Flow)
 					delete(fc.flowsToProcessReconcile, flowId)
 				}
 			}
@@ -2434,12 +2541,7 @@ func (fc *FlowCollector) reconcileConnectorRecords() error {
 				siteId := fc.getRecordSiteId(*connector)
 				var matchHost *string
 				found := false
-				if net.ParseIP(*connector.DestHost) == nil {
-					addrs, err := net.LookupHost(*connector.DestHost)
-					if err == nil && len(addrs) > 0 {
-						matchHost = &addrs[0]
-					}
-				} else {
+				if net.ParseIP(*connector.DestHost) != nil {
 					matchHost = connector.DestHost
 				}
 				for _, process := range fc.Processes {
@@ -2458,7 +2560,9 @@ func (fc *FlowCollector) reconcileConnectorRecords() error {
 							connector.Target = process.Name
 							process.connector = &connector.Identity
 							process.ProcessBinding = &Bound
-							fc.updateNetworkStatus()
+							if fc.mode == RecordStatus {
+								fc.updateNetworkStatus()
+							}
 							log.Printf("COLLECTOR: Connector %s/%s associated to process %s\n", connector.Identity, *connector.Address, *process.Name)
 							delete(fc.connectorsToReconcile, connId)
 							break
@@ -2479,6 +2583,8 @@ func (fc *FlowCollector) reconcileConnectorRecords() error {
 						for _, process := range fc.Processes {
 							if process.Name != nil && *process.Name == processName {
 								log.Printf("COLLECTOR: Associating connector %s to external process %s\n", connector.Identity, processName)
+								process.ProcessBinding = &Bound
+								process.connector = &connector.Identity
 								connector.ProcessId = &process.Identity
 								delete(fc.connectorsToReconcile, connId)
 								break
@@ -2688,3 +2794,117 @@ func (fc *FlowCollector) needForSiteProcess(flow *FlowRecord, siteId string, sta
 	}
 	return found
 }
+
+// graph relations between routers and sites using LinkRecords
+//
+// Collector state is fallible, as such this graph is meant to be conservative
+// in the edges it includes between nodes. It de-duplicates links (see
+// skupperproject/skupper-router issue #1456) and ensures both the listener and
+// connector sides of inter-router links are present representing either.
+func (fc *FlowCollector) graph() (routers, sites map[string]*node) {
+	siteByRouterID := make(map[string]string, len(fc.Routers))
+	routerIDByName := make(map[string]string, len(fc.Routers))
+	for _, router := range fc.Routers {
+		if _, ok := fc.Sites[router.Parent]; !ok || router.Name == nil {
+			continue
+		}
+		siteByRouterID[router.Identity] = router.Parent
+		routerIDByName[normalizeRouterName(*router.Name)] = router.Identity
+	}
+	setLink := func(set map[string]map[string]string, local, peer, role string) {
+		s, ok := set[local]
+		if !ok {
+			set[local] = make(map[string]string)
+			s = set[local]
+		}
+		s[peer] = role
+	}
+	// find unique links by localRotuerID and peerRouterID
+	outgoing := make(map[string]map[string]string)
+	incoming := make(map[string]map[string]string)
+	for _, link := range fc.Links {
+		if link.Direction == nil || link.Name == nil {
+			continue
+		}
+		localRouterID, peerRouterName := link.Parent, *link.Name
+
+		linkRole := "inter-router"
+		if link.Mode != nil && *link.Mode != "" {
+			linkRole = *link.Mode
+		}
+
+		if _, ok := fc.Routers[localRouterID]; !ok {
+			continue
+		}
+		peerRouterID, ok := routerIDByName[peerRouterName]
+		if !ok {
+			continue
+		}
+		switch *link.Direction {
+		case "incoming":
+			setLink(incoming, localRouterID, peerRouterID, linkRole)
+		case "outgoing":
+			setLink(outgoing, localRouterID, peerRouterID, linkRole)
+		}
+	}
+
+	routerNodes := make(map[string]*node, len(fc.Routers))
+	for id := range fc.Routers {
+		routerNodes[id] = &node{ID: id}
+	}
+	siteNodes := make(map[string]*node, len(fc.Sites))
+	for id := range fc.Sites {
+		siteNodes[id] = &node{ID: id}
+	}
+
+	for router, links := range outgoing {
+		for peerRouter, role := range links {
+			switch role {
+			// Since the router does not presently include Link records for
+			// edge router connections on the listener side, for edge router
+			// links we include all unique links on the connector (Outgoing)
+			// side.
+			case "edge":
+				site, peerSite := siteByRouterID[router], siteByRouterID[peerRouter]
+				rNode, siteNode := routerNodes[router], siteNodes[site]
+				if rNode == nil || siteNode == nil {
+					continue
+				}
+				rNode.Forward = append(rNode.Forward, peerRouter)
+				if site != peerSite {
+					siteNode.Forward = append(siteNode.Forward, peerSite)
+				}
+			default:
+				fallthrough
+			case "inter-router":
+				// Ignore inter-router links only when the connector side
+				// exists but not the listener side or vice-versa. When one or
+				// the other of these records is missing it is likely that the
+				// connection is down.
+				peerLinks, ok := incoming[peerRouter]
+				if !ok {
+					continue
+				}
+				if _, ok := peerLinks[router]; !ok {
+					continue
+				}
+				site, peerSite := siteByRouterID[router], siteByRouterID[peerRouter]
+				rNode, peerNode := routerNodes[router], routerNodes[peerRouter]
+				rNode.Forward = append(rNode.Forward, peerRouter)
+				peerNode.Backward = append(peerNode.Backward, router)
+				if site != peerSite {
+					siteNode, peerSiteNode := siteNodes[site], siteNodes[peerSite]
+					siteNode.Forward = append(siteNode.Forward, peerSite)
+					peerSiteNode.Backward = append(peerSiteNode.Backward, site)
+				}
+			}
+		}
+	}
+	return routerNodes, siteNodes
+}
+
+type node struct {
+	ID       string
+	Forward  []string
+	Backward []string
+}
