@@ -1,14 +1,20 @@
 package flow
 
 import (
+	"context"
 	"encoding/json"
 	"net/http"
+	"strings"
 	"testing"
 	"time"
 
 	"github.com/gorilla/mux"
 	"github.com/prometheus/client_golang/prometheus"
+	"github.com/skupperproject/skupper/api/types"
 	"gotest.tools/assert"
+	v1 "k8s.io/api/core/v1"
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
+	"k8s.io/client-go/kubernetes/fake"
 )
 
 func TestRecordGraphWithMetrics(t *testing.T) {
@@ -22,20 +28,31 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 	address2 := "redis-cart"
 	address3 := "mongo"
 	address4 := "cartservice"
+	addressExt := "svc-acme-foo"
 	protocol := "tcp"
-	counterFlow1 := "flow:0"
 	counterFlow2 := "flow:2"
 	counterFlow4 := "flow:4"
-	flowTrace := "router:0|router:1"
+	counterFlow10 := "flow:10"
 	processName1 := "checkout-1234"
 	processName2 := "payment-1234"
 	groupName1 := "online-store"
-	listenerProcess := "process:0"
 	connectorProcess1 := "process:1"
 	connectorProcess2 := "process:2"
 	processConnector := "connector:0"
+	sourceHost1 := "11.22.33.44"
 	destHost1 := "10.20.30.40"
 	destHost2 := "host.cloud.com"
+	destHostExt := "172.4.4.8"
+	connectorHost := "0.0.0.0"
+	linkNames := []string{
+		"site1.1",
+		"site2.0",
+	}
+	routerNames := []string{
+		"0/site1.0",
+		"0/site1.1",
+		"0/site2.0",
+	}
 
 	sites := []SiteRecord{
 		{
@@ -67,7 +84,7 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 				Parent:    "site:0",
 				StartTime: uint64(time.Now().UnixNano()) / uint64(time.Microsecond),
 			},
-			Name: &name,
+			Name: &routerNames[0],
 		},
 		{
 			Base: Base{
@@ -76,7 +93,16 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 				Parent:    "site:0",
 				StartTime: uint64(time.Now().UnixNano()) / uint64(time.Microsecond),
 			},
-			Name: &name,
+			Name: &routerNames[1],
+		},
+		{
+			Base: Base{
+				RecType:   recordNames[Router],
+				Identity:  "router:2",
+				Parent:    "site:1",
+				StartTime: uint64(time.Now().UnixNano()) / uint64(time.Microsecond),
+			},
+			Name: &routerNames[2],
 		},
 	}
 	hosts := []HostRecord{
@@ -110,17 +136,21 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 			Base: Base{
 				RecType:   recordNames[Link],
 				Identity:  "link:0",
-				Parent:    "router:0",
+				Parent:    "router:1",
 				StartTime: uint64(time.Now().UnixNano()) / uint64(time.Microsecond),
 			},
+			Name:      &linkNames[0],
+			Direction: &Incoming,
 		},
 		{
 			Base: Base{
 				RecType:   recordNames[Link],
 				Identity:  "link:1",
-				Parent:    "router:1",
+				Parent:    "router:2",
 				StartTime: uint64(time.Now().UnixNano()) / uint64(time.Microsecond),
 			},
+			Name:      &linkNames[1],
+			Direction: &Outgoing,
 		},
 	}
 	processes := []ProcessRecord{
@@ -131,8 +161,9 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 				Parent:    "site:0",
 				StartTime: uint64(time.Now().UnixNano()) / uint64(time.Microsecond),
 			},
-			Name:      &processName1,
-			GroupName: &groupName1,
+			Name:       &processName1,
+			GroupName:  &groupName1,
+			SourceHost: &sourceHost1,
 		},
 		{
 			Base: Base{
@@ -191,6 +222,16 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 			Protocol: &protocol,
 			Address:  &address2,
 		},
+		{
+			Base: Base{
+				RecType:   recordNames[Listener],
+				Identity:  "listener:3",
+				Parent:    "router:1",
+				StartTime: uint64(time.Now().UnixNano()) / uint64(time.Microsecond),
+			},
+			Protocol: &protocol,
+			Address:  &addressExt,
+		},
 	}
 	connectors := []ConnectorRecord{
 		{
@@ -238,7 +279,22 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 			ProcessId: &connectorProcess1,
 			DestHost:  &destHost1,
 		},
+		{
+			Base: Base{
+				RecType:   recordNames[Connector],
+				Identity:  "connector:3",
+				Parent:    "router:0",
+				StartTime: uint64(time.Now().Add(-3*time.Minute).UnixNano()) / uint64(time.Microsecond),
+			},
+			Protocol: &protocol,
+			Address:  &addressExt,
+			DestHost: &destHostExt,
+		},
 	}
+	// Two very screwy flow pairs
+	//	flow:2/flow:3 a regular tcp flow pair from process:0 -> process:1
+	//  flow:4/flow:5 a L7 flow pair with parents flow:0/flow:1. Also from process:0 -> process:1 with protocol tcp
+	//  un-paired flow:0/flow:1 act as parents for the L7 flow pair. Mostly nonsense, as they reference l4 listeners and connectors
 	flows := []FlowRecord{
 		{
 			Base: Base{
@@ -247,8 +303,7 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 				Parent:    "listener:0",
 				StartTime: uint64(time.Now().UnixNano()) / uint64(time.Microsecond),
 			},
-			ProcessName: &processName1,
-			Process:     &listenerProcess,
+			SourceHost: &sourceHost1,
 		},
 		{
 			Base: Base{
@@ -257,11 +312,7 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 				Parent:    "connector:0",
 				StartTime: uint64(time.Now().UnixNano()) / uint64(time.Microsecond),
 			},
-			CounterFlow: &counterFlow1,
-			Trace:       &flowTrace,
-			ProcessName: &processName2,
-			Process:     &connectorProcess1,
-			SourceHost:  &destHost1,
+			SourceHost: &connectorHost,
 		},
 		{
 			Base: Base{
@@ -270,8 +321,7 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 				Parent:    "listener:0",
 				StartTime: uint64(time.Now().UnixNano()) / uint64(time.Microsecond),
 			},
-			ProcessName: &processName1,
-			Process:     &listenerProcess,
+			SourceHost: &sourceHost1,
 		},
 		{
 			Base: Base{
@@ -280,6 +330,7 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 				Parent:    "connector:0",
 				StartTime: uint64(time.Now().UnixNano()) / uint64(time.Microsecond),
 			},
+			SourceHost:  &connectorHost,
 			CounterFlow: &counterFlow2,
 			ProcessName: &processName2,
 			Process:     &connectorProcess1,
@@ -292,6 +343,7 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 				StartTime: uint64(time.Now().UnixNano()) / uint64(time.Microsecond),
 				EndTime:   uint64(time.Now().UnixNano()) / uint64(time.Microsecond),
 			},
+			SourceHost:  &sourceHost1,
 			ProcessName: &processName1,
 		},
 		{
@@ -301,8 +353,6 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 				Parent:    "flow:0",
 				StartTime: uint64(time.Now().UnixNano()) / uint64(time.Microsecond),
 			},
-			ProcessName: &processName1,
-			Process:     &listenerProcess,
 		},
 		{
 			Base: Base{
@@ -312,12 +362,34 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 				StartTime: uint64(time.Now().UnixNano()) / uint64(time.Microsecond),
 			},
 			CounterFlow: &counterFlow4,
-			ProcessName: &processName2,
-			Trace:       &flowTrace,
-			Process:     &connectorProcess1,
 		},
 	}
 
+	// An additional tcp flow pair (10,11):
+	// Exchange between process:0 -> an unknown external process. Should
+	// introduce a site-server process
+	flows = append(flows, []FlowRecord{
+		{
+			Base: Base{
+				RecType:   recordNames[Flow],
+				Identity:  "flow:10",
+				Parent:    "listener:3",
+				StartTime: uint64(time.Now().UnixNano()) / uint64(time.Microsecond),
+			},
+			SourceHost: &sourceHost1,
+		},
+		{
+			Base: Base{
+				RecType:   recordNames[Flow],
+				Identity:  "flow:11",
+				Parent:    "connector:3",
+				StartTime: uint64(time.Now().UnixNano()) / uint64(time.Microsecond),
+			},
+			SourceHost:  &connectorHost,
+			CounterFlow: &counterFlow10,
+		},
+	}...)
+
 	reg := prometheus.NewRegistry()
 	u, _ := time.ParseDuration("5m")
 	fc := NewFlowCollector(FlowCollectorSpec{
@@ -327,6 +399,7 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 		ConnectionFactory: nil,
 		FlowRecordTtl:     u,
 	})
+	fc.startTime = uint64(time.Now().Add(time.Minute * -5).UnixMicro())
 	fc.metrics = fc.NewMetrics(fc.prometheusReg)
 	for _, s := range sites {
 		err := fc.updateRecord(s)
@@ -358,6 +431,12 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 		err = fc.updateRecord(p)
 		assert.Assert(t, err)
 	}
+	for _, f := range flows {
+		err := fc.updateRecord(f)
+		assert.Assert(t, err)
+		err = fc.updateRecord(f)
+		assert.Assert(t, err)
+	}
 	for _, l := range listeners {
 		err := fc.updateRecord(l)
 		assert.Assert(t, err)
@@ -370,13 +449,6 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 		err = fc.updateRecord(c)
 		assert.Assert(t, err)
 	}
-	for _, f := range flows {
-		err := fc.updateRecord(f)
-		assert.Assert(t, err)
-		err = fc.updateRecord(f)
-		assert.Assert(t, err)
-	}
-
 	for _, s := range sites {
 		id := fc.getRecordSiteId(s)
 		assert.Equal(t, id, s.Identity)
@@ -391,7 +463,11 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 	}
 	for _, l := range links {
 		id := fc.getRecordSiteId(l)
-		assert.Equal(t, id, "site:0")
+		expected := "site:0"
+		if l.Parent == "router:2" {
+			expected = "site:1"
+		}
+		assert.Equal(t, id, expected)
 	}
 	for _, p := range processes {
 		id := fc.getRecordSiteId(p)
@@ -412,7 +488,7 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 		}
 		if f.Trace != nil {
 			trace := fc.annotateFlowTrace(&f)
-			assert.Equal(t, *trace, "skupper-site1@skupper-site1")
+			assert.Equal(t, *trace, "site1.0@skupper-site1")
 		}
 	}
 
@@ -428,11 +504,16 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 	}
 
 	var processGroupId string
-	for x, _ := range fc.ProcessGroups {
-		processGroupId = x
-		break
+	for x, group := range fc.ProcessGroups {
+		if group.Name != nil && !strings.HasPrefix(*group.Name, "site-servers") {
+			processGroupId = x
+			break
+		}
 	}
 
+	fc.reconcileConnectorRecords()
+	fc.reconcileFlowRecords()
+
 	type test struct {
 		desc         string
 		recordType   int
@@ -473,7 +554,7 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 			params:       map[string]string{"sortBy": "identity.asc"},
 			vars:         map[string]string{"id": "site:0"},
 			name:         "routers",
-			responseSize: len(routers),
+			responseSize: 2,
 		},
 		{
 			desc:         "Get Site links",
@@ -483,7 +564,7 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 			params:       map[string]string{"sortBy": "identity.asc"},
 			vars:         map[string]string{"id": "site:0"},
 			name:         "links",
-			responseSize: len(links),
+			responseSize: 1,
 		},
 		{
 			desc:         "Get Site hosts",
@@ -503,7 +584,7 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 			params:       map[string]string{"sortBy": "identity.asc"},
 			vars:         map[string]string{"id": "site:0"},
 			name:         "processes",
-			responseSize: len(processes),
+			responseSize: len(processes) + 1,
 		},
 		{
 			desc:         "Get Host list",
@@ -553,7 +634,7 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 			params:       map[string]string{"sortBy": "identity.asc"},
 			vars:         map[string]string{"id": "router:0"},
 			name:         "flows",
-			responseSize: 6,
+			responseSize: 7,
 		},
 		{
 			desc:         "Get Router links",
@@ -561,7 +642,7 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 			method:       "Get",
 			url:          "/",
 			params:       map[string]string{"sortBy": "identity.asc"},
-			vars:         map[string]string{"id": "router:0"},
+			vars:         map[string]string{"id": "router:1"},
 			name:         "links",
 			responseSize: 1,
 		},
@@ -583,7 +664,7 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 			params:       map[string]string{"sortBy": "identity.asc"},
 			vars:         map[string]string{"id": "router:0"},
 			name:         "connectors",
-			responseSize: 2,
+			responseSize: 3,
 		},
 		{
 			desc:         "Get Link list",
@@ -643,7 +724,7 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 			params:       map[string]string{"sortBy": "identity.asc"},
 			vars:         map[string]string{},
 			name:         "list",
-			responseSize: 3,
+			responseSize: 4,
 		},
 		{
 			desc:         "Get Connector item",
@@ -713,7 +794,7 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 			params:       map[string]string{"sortBy": "identity.asc"},
 			vars:         map[string]string{"id": addressId},
 			name:         "flowpairs",
-			responseSize: 3,
+			responseSize: 2,
 		},
 		{
 			desc:         "Get Address listeners",
@@ -781,7 +862,7 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 			method:       "Get",
 			url:          "/",
 			params:       map[string]string{"sortBy": "identity.asc"},
-			vars:         map[string]string{"id": "fp-flow:0"},
+			vars:         map[string]string{"id": "fp-flow:2"},
 			name:         "item",
 			responseSize: 1,
 		},
@@ -793,7 +874,17 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 			params:       map[string]string{"sortBy": "identity.asc"},
 			vars:         map[string]string{},
 			name:         "list",
-			responseSize: len(processes),
+			responseSize: len(processes) + 1,
+		},
+		{
+			desc:         "Get Process site-servers",
+			recordType:   Process,
+			method:       "Get",
+			url:          "/",
+			params:       map[string]string{"sortBy": "identity.asc", "name": "site-servers", "processBinding": "bound"},
+			vars:         map[string]string{},
+			name:         "list",
+			responseSize: 1,
 		},
 		{
 			desc:         "Get Process item",
@@ -813,7 +904,7 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 			params:       map[string]string{"sortBy": "identity.asc"},
 			vars:         map[string]string{"id": "process:0"},
 			name:         "flows",
-			responseSize: 3,
+			responseSize: 4,
 		},
 		{
 			desc:         "Get Process addresses",
@@ -843,7 +934,7 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 			params:       map[string]string{"sortBy": "identity.asc"},
 			vars:         map[string]string{},
 			name:         "list",
-			responseSize: 1,
+			responseSize: 2,
 		},
 		{
 			desc:         "Get ProcessGroup item",
@@ -893,7 +984,7 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 			params:       map[string]string{"sortBy": "identity.asc"},
 			vars:         map[string]string{},
 			name:         "list",
-			responseSize: 1,
+			responseSize: 2,
 		},
 		{
 			desc:         "Get ProcessPair list",
@@ -903,7 +994,7 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 			params:       map[string]string{"sortBy": "identity.asc"},
 			vars:         map[string]string{},
 			name:         "list",
-			responseSize: 1,
+			responseSize: 2,
 		},
 		{
 			desc:         "Get ProcessPair item",
@@ -937,9 +1028,6 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 		},
 	}
 
-	fc.reconcileConnectorRecords()
-	fc.reconcileFlowRecords()
-
 	var payload Payload
 	for _, test := range testTable {
 		req, _ := http.NewRequest(test.method, test.url, nil)
@@ -956,6 +1044,10 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 		assert.Equal(t, test.responseSize, payload.Count, test.desc)
 	}
 
+	for _, flow := range fc.Flows {
+		assert.DeepEqual(t, flow.Protocol, &protocol)
+	}
+
 	endTime := uint64(time.Now().UnixNano()) / uint64(time.Microsecond)
 	for _, c := range connectors {
 		c.EndTime = endTime
@@ -994,8 +1086,7 @@ func TestRecordGraphWithMetrics(t *testing.T) {
 	}
 }
 
-// TODO: address dependence on cli
-func xTestRecordGraphNetworkStatus(t *testing.T) {
+func TestRecordGraphNetworkStatus(t *testing.T) {
 	name := "skupper-site"
 	namespace := "skupper-public"
 	provider := "aws"
@@ -1186,14 +1277,26 @@ func xTestRecordGraphNetworkStatus(t *testing.T) {
 	}
 
 	u, _ := time.ParseDuration("5m")
+	client := fake.NewSimpleClientset()
+	configMapClient := client.CoreV1().ConfigMaps("default")
+	_, err := configMapClient.Create(
+		context.Background(),
+		&v1.ConfigMap{ObjectMeta: metav1.ObjectMeta{Name: types.NetworkStatusConfigMapName}},
+		metav1.CreateOptions{},
+	)
+	if err != nil {
+		t.Fatalf("kube cliet setup failed: %v", err)
+	}
 	fc := NewFlowCollector(FlowCollectorSpec{
-		Mode:              RecordStatus,
-		Namespace:         "default",
-		Origin:            "origin",
-		PromReg:           nil,
-		ConnectionFactory: nil,
-		FlowRecordTtl:     u,
+		Mode:                RecordStatus,
+		Namespace:           "default",
+		Origin:              "origin",
+		PromReg:             nil,
+		ConnectionFactory:   nil,
+		FlowRecordTtl:       u,
+		NetworkStatusClient: client,
 	})
+
 	for _, s := range sites {
 		err := fc.updateRecord(s)
 		assert.Assert(t, err)
@@ -1268,6 +1371,255 @@ func xTestRecordGraphNetworkStatus(t *testing.T) {
 
 	fc.reconcileConnectorRecords()
 
-	err := fc.updateNetworkStatus()
-	assert.Assert(t, err)
+	fc.updateNetworkStatus()
+
+	cm, err := configMapClient.Get(context.Background(), types.NetworkStatusConfigMapName, metav1.GetOptions{})
+	if err != nil {
+		t.Fatal("expected configmap", err)
+	}
+	networkStatus, ok := cm.Data["NetworkStatus"]
+	assert.Check(t, ok)
+	var status NetworkStatus
+	assert.Check(t, json.Unmarshal([]byte(networkStatus), &status))
+	assert.Equal(t, len(status.Sites), len(sites))
+	assert.Equal(t, len(status.Addresses), 4)
+
+}
+
+func TestGraph(t *testing.T) {
+	newSite := func(id string) *SiteRecord {
+		return &SiteRecord{
+			Base: Base{
+				RecType:  recordNames[Site],
+				Identity: id,
+			},
+		}
+	}
+	newRouter := func(id string, parent string, name string) *RouterRecord {
+		rn := "0/" + name
+		return &RouterRecord{Base: Base{
+			RecType:  recordNames[Router],
+			Identity: id,
+			Parent:   parent,
+		},
+			Name: &rn,
+		}
+	}
+
+	type largs struct {
+		ID             string
+		RouterID       string
+		PeerRouterName string
+		Direction      string
+		Role           string
+	}
+	newLink := func(a largs) *LinkRecord {
+		record := &LinkRecord{
+			Base: Base{
+				RecType:  recordNames[Router],
+				Identity: a.ID,
+				Parent:   a.RouterID,
+			},
+			Name:      &a.PeerRouterName,
+			Direction: &a.Direction,
+		}
+		if a.Role != "" {
+			record.Mode = &a.Role
+		}
+		return record
+	}
+	tests := []struct {
+		Name                string
+		Sites               []*SiteRecord
+		Routers             []*RouterRecord
+		Links               []*LinkRecord
+		ExpectedRouterNodes map[string]*node
+		ExpectedSiteNodes   map[string]*node
+	}{
+		{
+			Name: "empty",
+		}, {
+			Name: "missing routers",
+			Sites: []*SiteRecord{
+				newSite("site:0"), newSite("site:1"),
+			},
+			Links: []*LinkRecord{
+				newLink(largs{ID: "norouter:0", RouterID: "router:0", PeerRouterName: "r1", Direction: Incoming}),
+				newLink(largs{ID: "norouter:1", RouterID: "router:1", PeerRouterName: "r0", Direction: Outgoing}),
+				newLink(largs{ID: "norouter:2", RouterID: "router:2", PeerRouterName: "r0", Direction: Outgoing, Role: "edge"}),
+			},
+			ExpectedSiteNodes: map[string]*node{
+				"site:0": {ID: "site:0"},
+				"site:1": {ID: "site:1"},
+			},
+		}, {
+			Name: "missing site",
+			Sites: []*SiteRecord{
+				newSite("site:0"),
+			},
+			Routers: []*RouterRecord{
+				newRouter("router0", "site:0", "r0"),
+				newRouter("router1", "site:1", "r1"),
+				newRouter("router2", "site:2", "r2"),
+			},
+			Links: []*LinkRecord{
+				newLink(largs{ID: "link0", RouterID: "router0", PeerRouterName: "r1", Direction: Incoming}),
+				newLink(largs{ID: "link1", RouterID: "router1", PeerRouterName: "r0", Direction: Outgoing}),
+				newLink(largs{ID: "link2", RouterID: "router2", PeerRouterName: "r0", Direction: Outgoing, Role: "edge"}),
+			},
+			ExpectedRouterNodes: map[string]*node{
+				"router0": {ID: "router0"},
+				"router1": {ID: "router1"},
+				"router2": {ID: "router2"},
+			},
+			ExpectedSiteNodes: map[string]*node{
+				"site:0": {ID: "site:0"},
+			},
+		}, {
+			Name: "single link pair",
+			Sites: []*SiteRecord{
+				newSite("site:0"), newSite("site:1"),
+			},
+			Routers: []*RouterRecord{
+				newRouter("router0", "site:0", "r0"),
+				newRouter("router1", "site:1", "r1"),
+			},
+			Links: []*LinkRecord{
+				newLink(largs{ID: "link0", RouterID: "router0", PeerRouterName: "r1", Direction: Incoming}),
+				newLink(largs{ID: "link1", RouterID: "router1", PeerRouterName: "r0", Direction: Outgoing}),
+			},
+			ExpectedRouterNodes: map[string]*node{
+				"router0": {ID: "router0", Backward: []string{"router1"}},
+				"router1": {ID: "router1", Forward: []string{"router0"}},
+			},
+			ExpectedSiteNodes: map[string]*node{
+				"site:0": {ID: "site:0", Backward: []string{"site:1"}},
+				"site:1": {ID: "site:1", Forward: []string{"site:0"}},
+			},
+		}, {
+			Name: "redundant links",
+			Sites: []*SiteRecord{
+				newSite("site:0"), newSite("site:1"),
+			},
+			Routers: []*RouterRecord{
+				newRouter("router0", "site:0", "r0"),
+				newRouter("router1", "site:1", "r1"),
+			},
+			Links: []*LinkRecord{
+				newLink(largs{ID: "link0", RouterID: "router0", PeerRouterName: "r1", Direction: Incoming}),
+				newLink(largs{ID: "link1", RouterID: "router1", PeerRouterName: "r0", Direction: Outgoing}),
+				newLink(largs{ID: "link2", RouterID: "router0", PeerRouterName: "r1", Direction: Incoming}),
+				newLink(largs{ID: "link3", RouterID: "router1", PeerRouterName: "r0", Direction: Outgoing}),
+				newLink(largs{ID: "link80", RouterID: "router0", PeerRouterName: "r1", Direction: Incoming}),
+				newLink(largs{ID: "link90", RouterID: "router0", PeerRouterName: "r1", Direction: Incoming}),
+			},
+			ExpectedRouterNodes: map[string]*node{
+				"router0": {ID: "router0", Backward: []string{"router1"}},
+				"router1": {ID: "router1", Forward: []string{"router0"}},
+			},
+			ExpectedSiteNodes: map[string]*node{
+				"site:0": {ID: "site:0", Backward: []string{"site:1"}},
+				"site:1": {ID: "site:1", Forward: []string{"site:0"}},
+			},
+		}, {
+			Name: "complex",
+			Sites: []*SiteRecord{
+				newSite("site:0"), newSite("site:1"), newSite("site:2"),
+			},
+			Routers: []*RouterRecord{
+				newRouter("router0", "site:0", "r0"),
+				newRouter("router1", "site:1", "r1"),
+				newRouter("router2", "site:2", "r2"),
+			},
+			Links: []*LinkRecord{
+				newLink(largs{ID: "link0", RouterID: "router1", PeerRouterName: "r0", Direction: Outgoing}), // half link to router1 -> router0
+				newLink(largs{ID: "link1", RouterID: "router1", PeerRouterName: "r2", Direction: Outgoing}), // |
+				newLink(largs{ID: "link2", RouterID: "router2", PeerRouterName: "r1", Direction: Incoming}), // | link pair router1,router2
+				newLink(largs{ID: "link3", RouterID: "router2", PeerRouterName: "r0", Direction: Incoming}), // half link to router2 -> router0
+				newLink(largs{ID: "bogus1", RouterID: "router2", PeerRouterName: "rdne", Direction: Incoming, Role: "edge"}),
+				newLink(largs{ID: "bogus2", RouterID: "routerdne", PeerRouterName: "r0", Direction: Incoming}),
+			},
+			ExpectedRouterNodes: map[string]*node{
+				"router0": {ID: "router0"},
+				"router1": {ID: "router1", Forward: []string{"router2"}},
+				"router2": {ID: "router2", Backward: []string{"router1"}},
+			},
+			ExpectedSiteNodes: map[string]*node{
+				"site:0": {ID: "site:0"},
+				"site:1": {ID: "site:1", Forward: []string{"site:2"}},
+				"site:2": {ID: "site:2", Backward: []string{"site:1"}},
+			},
+		}, {
+			Name: "edge links counted on connector side only",
+			Sites: []*SiteRecord{
+				newSite("site:0"), newSite("site:1"), newSite("site:2"),
+			},
+			Routers: []*RouterRecord{
+				newRouter("router0", "site:0", "r0"),
+				newRouter("router1", "site:1", "r1"),
+				newRouter("router2", "site:2", "r2"),
+			},
+			Links: []*LinkRecord{
+				newLink(largs{ID: "link1", RouterID: "router1", PeerRouterName: "r0", Direction: Outgoing, Role: "inter-router"}),
+				newLink(largs{ID: "link2", RouterID: "router2", PeerRouterName: "r0", Direction: Outgoing, Role: "edge"}),
+				newLink(largs{ID: "link-invalid", RouterID: "router0", PeerRouterName: "r2", Direction: Incoming, Role: "edge"}),
+			},
+			ExpectedRouterNodes: map[string]*node{
+				"router0": {ID: "router0"},
+				"router1": {ID: "router1"},
+				"router2": {ID: "router2", Forward: []string{"router0"}},
+			},
+			ExpectedSiteNodes: map[string]*node{
+				"site:0": {ID: "site:0"},
+				"site:1": {ID: "site:1"},
+				"site:2": {ID: "site:2", Forward: []string{"site:0"}},
+			},
+		}, {
+			Name: "exclude intra-site links",
+			Sites: []*SiteRecord{
+				newSite("site:0"),
+			},
+			Routers: []*RouterRecord{
+				newRouter("router0", "site:0", "r0"),
+				newRouter("router1", "site:0", "r1"),
+				newRouter("router2", "site:0", "r2"),
+			},
+			Links: []*LinkRecord{
+				newLink(largs{ID: "link1", RouterID: "router0", PeerRouterName: "r1", Direction: Incoming, Role: "inter-router"}),
+				newLink(largs{ID: "link2", RouterID: "router1", PeerRouterName: "r0", Direction: Outgoing, Role: "inter-router"}),
+				newLink(largs{ID: "link3", RouterID: "router2", PeerRouterName: "r0", Direction: Outgoing, Role: "edge"}),
+			},
+			ExpectedRouterNodes: map[string]*node{
+				"router0": {ID: "router0", Backward: []string{"router1"}},
+				"router1": {ID: "router1", Forward: []string{"router0"}},
+				"router2": {ID: "router2", Forward: []string{"router0"}},
+			},
+			ExpectedSiteNodes: map[string]*node{
+				"site:0": {ID: "site:0"},
+			},
+		},
+	}
+	for _, tc := range tests {
+		t.Run(tc.Name, func(t *testing.T) {
+			fc := NewFlowCollector(FlowCollectorSpec{})
+			for _, site := range tc.Sites {
+				fc.Sites[site.Identity] = site
+			}
+			for _, router := range tc.Routers {
+				fc.Routers[router.Identity] = router
+			}
+			for _, link := range tc.Links {
+				fc.Links[link.Identity] = link
+			}
+			if tc.ExpectedRouterNodes == nil {
+				tc.ExpectedRouterNodes = make(map[string]*node)
+			}
+			if tc.ExpectedSiteNodes == nil {
+				tc.ExpectedSiteNodes = make(map[string]*node)
+			}
+			routers, sites := fc.graph()
+			assert.DeepEqual(t, routers, tc.ExpectedRouterNodes)
+			assert.DeepEqual(t, sites, tc.ExpectedSiteNodes)
+		})
+	}
 }
